@@ -36,7 +36,7 @@ use crate::{
 /// See the [`Class`] trait for methods available on classes.
 /// See the [`Module`] trait for defining instance methods and nested
 /// classes/modules.
-/// See the [`Object`] trait for defining singlton methods (aka class methods).
+/// See the [`Object`] trait for defining singleton methods (aka class methods).
 ///
 /// See the [`ReprValue`] trait for additional methods available on this type.
 #[derive(Clone, Copy)]
@@ -120,18 +120,26 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*, RClass};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, RClass, Ruby};
     ///
-    /// let class = RClass::new(class::object()).unwrap();
-    /// assert!(class.is_kind_of(class::class()));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = RClass::new(ruby.class_object())?;
+    ///     assert!(class.is_kind_of(ruby.class_class()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{exception, prelude::*, ExceptionClass};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, ExceptionClass, Ruby};
     ///
-    /// assert!(ExceptionClass::new(exception::standard_error()).is_ok());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     assert!(ExceptionClass::new(ruby.exception_standard_error()).is_ok());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn new(superclass: Self) -> Result<Self, Error>;
 
@@ -141,70 +149,85 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let s = class::string().new_instance(()).unwrap();
-    /// assert!(s.is_kind_of(class::string()));
-    /// assert_eq!(s.to_string(), "");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.class_string().new_instance(())?;
+    ///     assert!(s.is_kind_of(ruby.class_string()));
+    ///     assert_eq!(s.to_string(), "");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{eval, kwargs, prelude::*, RClass};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, kwargs, prelude::*, Error, RClass, Ruby};
     ///
-    /// let cls: RClass = eval!(
-    ///     r#"
-    ///     class Foo
-    ///       def initialize(bar, baz:)
-    ///         @bar = bar
-    ///         @baz = baz
-    ///       end
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let cls: RClass = eval!(
+    ///         ruby,
+    ///         r#"
+    ///             class Foo
+    ///               def initialize(bar, baz:)
+    ///                 @bar = bar
+    ///                 @baz = baz
+    ///               end
     ///
-    ///       attr_reader(:bar, :baz)
-    ///     end
+    ///               attr_reader(:bar, :baz)
+    ///             end
     ///
-    ///     Object.const_get(:Foo)
-    /// "#
-    /// )
-    /// .unwrap();
-    /// let instance = cls.new_instance((1, kwargs!("baz" => 2))).unwrap();
-    /// assert!(instance.is_kind_of(cls));
-    /// let bar: i32 = instance.funcall("bar", ()).unwrap();
-    /// assert_eq!(bar, 1);
-    /// let baz: i32 = instance.funcall("baz", ()).unwrap();
-    /// assert_eq!(baz, 2);
+    ///             Object.const_get(:Foo)
+    ///         "#
+    ///     )?;
+    ///     let instance = cls.new_instance((1, kwargs!("baz" => 2)))?;
+    ///     assert!(instance.is_kind_of(cls));
+    ///     let bar: i32 = instance.funcall("bar", ())?;
+    ///     assert_eq!(bar, 1);
+    ///     let baz: i32 = instance.funcall("baz", ())?;
+    ///     assert_eq!(baz, 2);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{exception, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let s = exception::standard_error()
-    ///     .new_instance(("bang!",))
-    ///     .unwrap();
-    /// assert!(s.is_kind_of(exception::standard_error()));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.exception_standard_error().new_instance(("bang!",))?;
+    ///     assert!(s.is_kind_of(ruby.exception_standard_error()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{eval, ExceptionClass, kwargs, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{eval, ExceptionClass, kwargs, prelude::*, Error, Ruby};
     ///
-    /// let exc: ExceptionClass = eval!(
-    ///     r#"
-    ///     class MyError < StandardError
-    ///       def initialize(message:)
-    ///         super(message)
-    ///       end
-    ///     end
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let exc: ExceptionClass = eval!(
+    ///         ruby,
+    ///         r#"
+    ///             class MyError < StandardError
+    ///               def initialize(message:)
+    ///                 super(message)
+    ///               end
+    ///             end
     ///
-    ///     Object.const_get(:MyError)
-    /// "#
-    /// ).unwrap();
-    /// let s = exc.new_instance((kwargs!("message" => "bang!"),)).unwrap();
-    /// assert!(s.is_kind_of(exc));
-    /// let message: String = s.funcall("message", ()).unwrap();
-    /// assert_eq!(message, "bang!");
+    ///             Object.const_get(:MyError)
+    ///         "#
+    ///     )?;
+    ///     let s = exc.new_instance((kwargs!("message" => "bang!"),))?;
+    ///     assert!(s.is_kind_of(exc));
+    ///     let message: String = s.funcall("message", ())?;
+    ///     assert_eq!(message, "bang!");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn new_instance<T>(self, args: T) -> Result<Self::Instance, Error>
     where
@@ -216,19 +239,27 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let s = class::string().obj_alloc().unwrap();
-    /// assert!(s.is_kind_of(class::string()));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.class_string().obj_alloc()?;
+    ///     assert!(s.is_kind_of(ruby.class_string()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{exception, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let s = exception::standard_error().obj_alloc().unwrap();
-    /// assert!(s.is_kind_of(exception::standard_error()));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let s = ruby.exception_standard_error().obj_alloc()?;
+    ///     assert!(s.is_kind_of(ruby.exception_standard_error()));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn obj_alloc(self) -> Result<Self::Instance, Error>;
 
@@ -239,19 +270,27 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let klass = class::hash().superclass().unwrap();
-    /// assert!(klass.equal(class::object()).unwrap());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let klass = ruby.class_hash().superclass()?;
+    ///     assert!(klass.equal(ruby.class_object())?);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{class, exception, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let klass = exception::exception().superclass().unwrap();
-    /// assert!(klass.equal(class::object()).unwrap());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let klass = ruby.exception_exception().superclass()?;
+    ///     assert!(klass.equal(ruby.class_object())?);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn superclass(self) -> Result<RClass, Error> {
         protect(|| unsafe {
@@ -272,25 +311,33 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let value = class::hash();
-    /// // safe as we neve give Ruby a chance to free the string.
-    /// let s = unsafe { value.name() }.into_owned();
-    /// assert_eq!(s, "Hash");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let value = ruby.class_hash();
+    ///     // safe as we never give Ruby a chance to free the string.
+    ///     let s = unsafe { value.name() }.into_owned();
+    ///     assert_eq!(s, "Hash");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{exception, prelude::*};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, Error, Ruby};
     ///
-    /// let value = exception::standard_error();
-    /// // safe as we neve give Ruby a chance to free the string.
-    /// let s = unsafe { value.name() }.into_owned();
-    /// assert_eq!(s, "StandardError");
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let value = ruby.exception_standard_error();
+    ///     // safe as we never give Ruby a chance to free the string.
+    ///     let s = unsafe { value.name() }.into_owned();
+    ///     assert_eq!(s, "StandardError");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
-    unsafe fn name(&self) -> Cow<str> {
+    unsafe fn name(&self) -> Cow<'_, str> {
         let ptr = rb_class2name(self.as_rb_value());
         let cstr = CStr::from_ptr(ptr);
         cstr.to_string_lossy()
@@ -331,10 +378,7 @@ pub trait Class: Module {
     /// ```
     /// use std::cell::RefCell;
     ///
-    /// use magnus::{
-    ///     class, define_class, embed, eval, function, method, prelude::*, wrap, Error, RClass, Value,
-    /// };
-    /// # let _cleanup = unsafe { embed::init() };
+    /// use magnus::{function, method, prelude::*, wrap, Error, RClass, Ruby, Value};
     ///
     /// #[derive(Default)]
     /// struct Point {
@@ -370,34 +414,32 @@ pub trait Class: Module {
     ///     }
     /// }
     ///
-    /// let class = define_class("Point", class::object()).unwrap();
-    /// class.define_alloc_func::<MutPoint>();
-    /// class
-    ///     .define_singleton_method("create", function!(MutPoint::create, 2))
-    ///     .unwrap();
-    /// class
-    ///     .define_singleton_method("call_new", method!(MutPoint::call_new, 2))
-    ///     .unwrap();
-    /// class
-    ///     .define_method("initialize", method!(MutPoint::initialize, 2))
-    ///     .unwrap();
-    /// class
-    ///     .define_method("distance", method!(MutPoint::distance, 1))
-    ///     .unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = ruby.define_class("Point", ruby.class_object())?;
+    ///     class.define_alloc_func::<MutPoint>();
+    ///     class.define_singleton_method("create", function!(MutPoint::create, 2))?;
+    ///     class.define_singleton_method("call_new", method!(MutPoint::call_new, 2))?;
+    ///     class.define_method("initialize", method!(MutPoint::initialize, 2))?;
+    ///     class.define_method("distance", method!(MutPoint::distance, 1))?;
     ///
-    /// let d: f64 = eval(
-    ///     "class OffsetPoint < Point
-    ///        def initialize(offset, x, y)
-    ///          super(x + offset, y + offset)
-    ///        end
-    ///      end
-    ///      a = Point.new(1, 1)
-    ///      b = OffsetPoint.new(2, 3, 3)
-    ///      a.distance(b).round(2)",
-    /// )
-    /// .unwrap();
+    ///     let d: f64 = ruby.eval(
+    ///         "
+    ///           class OffsetPoint < Point
+    ///             def initialize(offset, x, y)
+    ///               super(x + offset, y + offset)
+    ///             end
+    ///           end
+    ///           a = Point.new(1, 1)
+    ///           b = OffsetPoint.new(2, 3, 3)
+    ///           a.distance(b).round(2)
+    ///         ",
+    ///     )?;
     ///
-    /// assert_eq!(d, 5.66);
+    ///     assert_eq!(d, 5.66);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn define_alloc_func<T>(self)
     where
@@ -424,15 +466,6 @@ pub trait Class: Module {
         }
     }
 
-    #[doc(hidden)]
-    #[deprecated(
-        since = "0.6.0",
-        note = "please use `undef_default_alloc_func` instead"
-    )]
-    fn undef_alloc_func(self) {
-        unsafe { rb_undef_alloc_func(self.as_rb_value()) }
-    }
-
     /// Remove the allocator function of a class if it is Ruby's default
     /// allocator function.
     ///
@@ -444,17 +477,22 @@ pub trait Class: Module {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{class, Class};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
-    /// let class = magnus::define_class("Point", class::object()).unwrap();
+    /// use magnus::{Class, Error, Ruby};
     ///
-    /// class.undef_default_alloc_func();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let class = ruby.define_class("Point", ruby.class_object())?;
     ///
-    /// let instance = class.new_instance(());
-    /// assert_eq!(
-    ///     "allocator undefined for Point",
-    ///     instance.err().unwrap().to_string()
-    /// );
+    ///     class.undef_default_alloc_func();
+    ///
+    ///     let instance = class.new_instance(());
+    ///     assert_eq!(
+    ///         "allocator undefined for Point",
+    ///         instance.err().unwrap().to_string()
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     fn undef_default_alloc_func(self) {
         static INIT: std::sync::Once = std::sync::Once::new();
@@ -466,10 +504,12 @@ pub trait Class: Module {
             });
             RB_CLASS_ALLOCATE_INSTANCE
         };
-
         unsafe {
-            if rb_get_alloc_func(self.as_rb_value()) == rb_class_allocate_instance {
-                rb_undef_alloc_func(self.as_rb_value())
+            let current_alloc = rb_get_alloc_func(self.as_rb_value());
+            if let (Some(actual), Some(default)) = (current_alloc, rb_class_allocate_instance) {
+                if std::ptr::eq(actual as *const (), default as *const ()) {
+                    rb_undef_alloc_func(self.as_rb_value());
+                }
             }
         }
     }
@@ -1174,9 +1214,10 @@ impl Ruby {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_array`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_array` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn array() -> RClass {
     get_ruby!().class_array()
@@ -1189,9 +1230,10 @@ pub fn array() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_basic_object`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_basic_object` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn basic_object() -> RClass {
     get_ruby!().class_basic_object()
@@ -1204,9 +1246,10 @@ pub fn basic_object() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_binding`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_binding` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn binding() -> RClass {
     get_ruby!().class_binding()
@@ -1219,9 +1262,10 @@ pub fn binding() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_class`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_class` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn class() -> RClass {
     get_ruby!().class_class()
@@ -1234,9 +1278,10 @@ pub fn class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_complex`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_complex` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn complex() -> RClass {
     get_ruby!().class_complex()
@@ -1249,9 +1294,10 @@ pub fn complex() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_dir`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_dir` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn dir() -> RClass {
     get_ruby!().class_dir()
@@ -1264,9 +1310,10 @@ pub fn dir() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_encoding`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_encoding` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn encoding() -> RClass {
     get_ruby!().class_encoding()
@@ -1279,9 +1326,10 @@ pub fn encoding() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_enumerator`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_enumerator` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn enumerator() -> RClass {
     get_ruby!().class_enumerator()
@@ -1294,9 +1342,10 @@ pub fn enumerator() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_false_class`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_false_class` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn false_class() -> RClass {
     get_ruby!().class_false_class()
@@ -1309,9 +1358,10 @@ pub fn false_class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_file`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_file` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn file() -> RClass {
     get_ruby!().class_file()
@@ -1324,9 +1374,10 @@ pub fn file() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_float`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_float` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn float() -> RClass {
     get_ruby!().class_float()
@@ -1339,9 +1390,10 @@ pub fn float() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_hash`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_hash` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn hash() -> RClass {
     get_ruby!().class_hash()
@@ -1354,9 +1406,10 @@ pub fn hash() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_io`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_io` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn io() -> RClass {
     get_ruby!().class_io()
@@ -1369,9 +1422,10 @@ pub fn io() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_integer`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_integer` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn integer() -> RClass {
     get_ruby!().class_integer()
@@ -1384,9 +1438,10 @@ pub fn integer() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_match`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_match` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn match_class() -> RClass {
     get_ruby!().class_match()
@@ -1399,9 +1454,10 @@ pub fn match_class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_method`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_method` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn method() -> RClass {
     get_ruby!().class_method()
@@ -1414,9 +1470,10 @@ pub fn method() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_module`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_module` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn module() -> RClass {
     get_ruby!().class_module()
@@ -1429,9 +1486,10 @@ pub fn module() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_name_error_mesg`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_name_error_mesg` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn name_error_mesg() -> RClass {
     get_ruby!().class_name_error_mesg()
@@ -1444,9 +1502,10 @@ pub fn name_error_mesg() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_nil_class`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_nil_class` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn nil_class() -> RClass {
     get_ruby!().class_nil_class()
@@ -1459,9 +1518,10 @@ pub fn nil_class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_numeric`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_numeric` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn numeric() -> RClass {
     get_ruby!().class_numeric()
@@ -1474,9 +1534,10 @@ pub fn numeric() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_object`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_object` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn object() -> RClass {
     get_ruby!().class_object()
@@ -1489,9 +1550,10 @@ pub fn object() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_proc`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_proc` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn proc() -> RClass {
     get_ruby!().class_proc()
@@ -1504,9 +1566,10 @@ pub fn proc() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_random`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_random` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn random() -> RClass {
     get_ruby!().class_random()
@@ -1519,9 +1582,10 @@ pub fn random() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_range`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_range` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn range() -> RClass {
     get_ruby!().class_range()
@@ -1534,9 +1598,10 @@ pub fn range() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_rational`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_rational` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn rational() -> RClass {
     get_ruby!().class_rational()
@@ -1549,9 +1614,10 @@ pub fn rational() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_refinement`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_refinement` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[cfg(any(ruby_gte_3_1, docsrs))]
 #[cfg_attr(docsrs, doc(cfg(ruby_gte_3_1)))]
 #[inline]
@@ -1566,9 +1632,10 @@ pub fn refinement() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_regexp`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_regexp` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn regexp() -> RClass {
     get_ruby!().class_regexp()
@@ -1581,9 +1648,10 @@ pub fn regexp() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_stat`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_stat` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn stat() -> RClass {
     get_ruby!().class_stat()
@@ -1596,9 +1664,10 @@ pub fn stat() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_string`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_string` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn string() -> RClass {
     get_ruby!().class_string()
@@ -1611,9 +1680,10 @@ pub fn string() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_struct`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_struct` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn struct_class() -> RClass {
     get_ruby!().class_struct()
@@ -1626,9 +1696,10 @@ pub fn struct_class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_symbol`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_symbol` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn symbol() -> RClass {
     get_ruby!().class_symbol()
@@ -1641,9 +1712,10 @@ pub fn symbol() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_thread`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_thread` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn thread() -> RClass {
     get_ruby!().class_thread()
@@ -1656,9 +1728,10 @@ pub fn thread() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_time`] for the
 /// non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_time` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn time() -> RClass {
     get_ruby!().class_time()
@@ -1671,9 +1744,10 @@ pub fn time() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_true_class`] for
 /// the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_true_class` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn true_class() -> RClass {
     get_ruby!().class_true_class()
@@ -1686,9 +1760,10 @@ pub fn true_class() -> RClass {
 /// Panics if called from a non-Ruby thread. See [`Ruby::class_unbound_method`]
 /// for the non-panicking version.
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::class_unbound_method` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn unbound_method() -> RClass {
     get_ruby!().class_unbound_method()
