@@ -5,12 +5,10 @@
 use std::{marker::PhantomData, ops::Range};
 
 use rb_sys::{
-    rb_gc_adjust_memory_usage, rb_gc_count, rb_gc_disable, rb_gc_enable, rb_gc_mark,
-    rb_gc_mark_locations, rb_gc_register_address, rb_gc_register_mark_object, rb_gc_start,
-    rb_gc_stat, rb_gc_unregister_address, VALUE,
+    rb_gc_adjust_memory_usage, rb_gc_count, rb_gc_disable, rb_gc_enable, rb_gc_location,
+    rb_gc_mark, rb_gc_mark_locations, rb_gc_mark_movable, rb_gc_register_address,
+    rb_gc_register_mark_object, rb_gc_start, rb_gc_stat, rb_gc_unregister_address, VALUE,
 };
-#[cfg(ruby_gte_2_7)]
-use rb_sys::{rb_gc_location, rb_gc_mark_movable};
 
 use crate::{
     error::{protect, Error},
@@ -20,7 +18,7 @@ use crate::{
     Ruby,
 };
 
-mod private {
+pub(crate) mod private {
     use super::*;
 
     pub trait Mark {
@@ -110,8 +108,6 @@ impl Marker {
     /// invalid to use from Rust when GC is run, you must update any stored
     /// objects with [`Compactor::location`] inside your implementation of
     /// [`DataTypeFunctions::compact`](`crate::typed_data::DataTypeFunctions::compact`).
-    #[cfg(any(ruby_gte_2_7, docsrs))]
-    #[cfg_attr(docsrs, doc(cfg(ruby_gte_2_7)))]
     pub fn mark_movable<T>(&self, value: T)
     where
         T: Mark,
@@ -168,15 +164,13 @@ impl Compactor {
     ///
     /// The [`Value`] type is effectly a pointer to a Ruby object. Ruby's
     /// garbage collector will avoid moving objects exposed to extensions,
-    /// unless the object has been marked with [`mark_movable`]. When
-    /// implementing
+    /// unless the object has been marked with
+    /// [`mark_movable`](Marker::mark_movable). When implementing
     /// [`DataTypeFunctions::compact`](`crate::typed_data::DataTypeFunctions::compact`)
     /// you will need to update any Ruby objects you are storing.
     ///
     /// Returns a new `T` that is pointing to the object that `value` used to
     /// point to. If `value` hasn't moved, simply returns `value`.
-    #[cfg(any(ruby_gte_2_7, docsrs))]
-    #[cfg_attr(docsrs, doc(cfg(ruby_gte_2_7)))]
     pub fn location<T>(&self, value: T) -> T
     where
         T: Locate,
@@ -185,45 +179,6 @@ impl Compactor {
     }
 }
 
-#[doc(hidden)]
-#[deprecated(since = "0.6.0", note = "please use `Marker::mark` instead")]
-pub fn mark<T>(value: T)
-where
-    T: ReprValue,
-{
-    Marker::new().mark(value)
-}
-
-#[doc(hidden)]
-#[deprecated(since = "0.6.0", note = "please use `Marker::mark_slice` instead")]
-pub fn mark_slice<T>(values: &[T])
-where
-    T: ReprValue,
-{
-    Marker::new().mark_slice(values)
-}
-
-#[doc(hidden)]
-#[deprecated(since = "0.6.0", note = "please use `Marker::mark_movable` instead")]
-#[cfg(any(ruby_gte_2_7, docsrs))]
-#[cfg_attr(docsrs, doc(cfg(ruby_gte_2_7)))]
-pub fn mark_movable<T>(value: T)
-where
-    T: ReprValue,
-{
-    Marker::new().mark_movable(value)
-}
-
-#[doc(hidden)]
-#[deprecated(since = "0.6.0", note = "please use `Compactor::location` instead")]
-#[cfg(any(ruby_gte_2_7, docsrs))]
-pub fn location<T>(value: T) -> T
-where
-    T: ReprValue,
-{
-    Compactor::new().location(value)
-}
-
 /// Registers `value` to never be garbage collected.
 ///
 /// This is essentially a deliberate memory leak.
@@ -231,16 +186,20 @@ where
 /// # Examples
 ///
 /// ```
-/// use magnus::{gc, RArray, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{gc, Error, Ruby};
 ///
-/// // will never be collected
-/// let root = RArray::new();
-/// gc::register_mark_object(root);
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     // will never be collected
+///     let root = ruby.ary_new();
+///     gc::register_mark_object(root);
 ///
-/// // won't be collected while it is in out `root` array
-/// let s = RString::new("example");
-/// root.push(s).unwrap();
+///     // won't be collected while it is in our `root` array
+///     let s = ruby.str_new("example");
+///     root.push(s).unwrap();
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn register_mark_object<T>(value: T)
 where
@@ -253,27 +212,31 @@ where
 ///
 /// Prevents Ruby moving or collecting `valref`. This should be used on
 /// `static` items to prevent them being collected instead of relying on Ruby
-/// constants/globals to allways refrence the value.
+/// constants/globals to always reference the value.
 ///
 /// See also [`BoxValue`](crate::value::BoxValue).
 ///
 /// # Examples
 ///
 /// ```
-/// use magnus::{gc, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{gc, Error, Ruby};
 ///
-/// let s = RString::new("example");
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let s = ruby.str_new("example");
 ///
-/// // s won't be collected even though it's on the heap
-/// let boxed = Box::new(s);
-/// gc::register_address(&*boxed);
+///     // s won't be collected even though it's on the heap
+///     let boxed = Box::new(s);
+///     gc::register_address(&*boxed);
 ///
-/// // ...
+///     // ...
 ///
-/// // allow s to be collected
-/// gc::unregister_address(&*boxed);
-/// drop(boxed);
+///     // allow s to be collected
+///     gc::unregister_address(&*boxed);
+///     drop(boxed);
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn register_address<T>(valref: &T)
 where
@@ -289,20 +252,24 @@ where
 /// # Examples
 ///
 /// ```
-/// use magnus::{gc, RString};
-/// # let _cleanup = unsafe { magnus::embed::init() };
+/// use magnus::{gc, Error, Ruby};
 ///
-/// let s = RString::new("example");
+/// fn example(ruby: &Ruby) -> Result<(), Error> {
+///     let s = ruby.str_new("example");
 ///
-/// // s won't be collected even though it's on the heap
-/// let boxed = Box::new(s);
-/// gc::register_address(&*boxed);
+///     // s won't be collected even though it's on the heap
+///     let boxed = Box::new(s);
+///     gc::register_address(&*boxed);
 ///
-/// // ...
+///     // ...
 ///
-/// // allow s to be collected
-/// gc::unregister_address(&*boxed);
-/// drop(boxed);
+///     // allow s to be collected
+///     gc::unregister_address(&*boxed);
+///     drop(boxed);
+///
+///     Ok(())
+/// }
+/// # Ruby::init(example).unwrap()
 /// ```
 pub fn unregister_address<T>(valref: &T)
 where
@@ -380,7 +347,7 @@ impl Ruby {
     /// Trigger a "full" GC run.
     ///
     /// This will perform a full mark phase and a complete sweep phase, but may
-    /// not run every single proceess associated with garbage collection.
+    /// not run every single process associated with garbage collection.
     ///
     /// Finalisers will be deferred to run later.
     ///
@@ -521,6 +488,7 @@ impl Ruby {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -534,9 +502,10 @@ impl Ruby {
 /// }
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_disable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn disable() -> bool {
     get_ruby!().gc_disable()
@@ -557,6 +526,7 @@ pub fn disable() -> bool {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -570,9 +540,10 @@ pub fn disable() -> bool {
 /// }
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_enable` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn enable() -> bool {
     get_ruby!().gc_enable()
@@ -581,7 +552,7 @@ pub fn enable() -> bool {
 /// Trigger a "full" GC run.
 ///
 /// This will perform a full mark phase and a complete sweep phase, but may not
-/// run every single proceess associated with garbage collection.
+/// run every single process associated with garbage collection.
 ///
 /// Finalisers will be deferred to run later.
 ///
@@ -596,15 +567,17 @@ pub fn enable() -> bool {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
 /// gc::start();
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_start` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn start() {
     get_ruby!().gc_start()
@@ -627,6 +600,7 @@ pub fn start() {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -640,9 +614,10 @@ pub fn start() {
 /// gc::adjust_memory_usage(-(mem_size as isize));
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_adjust_memory_usage` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn adjust_memory_usage(diff: isize) {
     get_ruby!().gc_adjust_memory_usage(diff)
@@ -659,6 +634,7 @@ pub fn adjust_memory_usage(diff: isize) {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -667,9 +643,10 @@ pub fn adjust_memory_usage(diff: isize) {
 /// assert!(gc::count() > before);
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_count` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn count() -> usize {
     get_ruby!().gc_count()
@@ -685,15 +662,17 @@ pub fn count() -> usize {
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::gc;
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
 /// assert!(gc::stat("heap_live_slots").unwrap() > 1);
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_stat` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn stat<T>(key: T) -> Result<usize, Error>
 where
@@ -713,6 +692,7 @@ where
 /// # Examples
 ///
 /// ```
+/// # #![allow(deprecated)]
 /// use magnus::{gc, Symbol};
 /// # let _cleanup = unsafe { magnus::embed::init() };
 ///
@@ -721,9 +701,10 @@ where
 /// assert!(live_slots > 1);
 /// ```
 #[cfg_attr(
-    not(feature = "friendly-api"),
+    not(feature = "old-api"),
     deprecated(note = "please use `Ruby::gc_all_stats` instead")
 )]
+#[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
 #[inline]
 pub fn all_stats() -> RHash {
     get_ruby!().gc_all_stats()
