@@ -13,7 +13,7 @@ use crate::{
         do_yield_iter, do_yield_splat_iter, do_yield_values_iter, Proc, Yield, YieldSplat,
         YieldValues,
     },
-    error::{raise, Error},
+    error::{raise, Error, IntoError},
     into_value::{ArgList, IntoValue},
     r_array::RArray,
     try_convert::TryConvert,
@@ -66,7 +66,7 @@ mod private {
         }
     }
 
-    seq!(N in 0..=16 {
+    seq!(N in 0..=15 {
         impl_method!(N);
     });
 
@@ -74,12 +74,15 @@ mod private {
         fn into_return_value(self) -> Result<Value, Error>;
     }
 
-    impl<T> ReturnValue for Result<T, Error>
+    impl<T, E> ReturnValue for Result<T, E>
     where
         T: IntoValue,
+        E: IntoError,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            self.map(|val| unsafe { val.into_value_unchecked() })
+            let ruby = unsafe { Ruby::get_unchecked() };
+            self.map(|val| val.into_value_with(&ruby))
+                .map_err(|err| err.into_error(&ruby))
         }
     }
 
@@ -88,7 +91,26 @@ mod private {
         T: IntoValue,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            Ok(self).into_return_value()
+            Ok::<T, Error>(self).into_return_value()
+        }
+    }
+
+    impl<I, T, E> ReturnValue for Result<Yield<I>, E>
+    where
+        I: Iterator<Item = T>,
+        T: IntoValue,
+        E: IntoError,
+    {
+        fn into_return_value(self) -> Result<Value, Error> {
+            let ruby = unsafe { Ruby::get_unchecked() };
+            self.map(|i| match i {
+                Yield::Iter(iter) => unsafe {
+                    do_yield_iter(iter);
+                    ruby.qnil().as_value()
+                },
+                Yield::Enumerator(e) => e.into_value_with(&ruby),
+            })
+            .map_err(|err| err.into_error(&ruby))
         }
     }
 
@@ -98,23 +120,26 @@ mod private {
         T: IntoValue,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            match self {
-                Yield::Iter(iter) => unsafe {
-                    do_yield_iter(iter);
-                    Ok(Ruby::get_unchecked().qnil().as_value())
-                },
-                Yield::Enumerator(e) => Ok(unsafe { e.into_value_unchecked() }),
-            }
+            Ok::<Self, Error>(self).into_return_value()
         }
     }
 
-    impl<I, T> ReturnValue for Result<Yield<I>, Error>
+    impl<I, T, E> ReturnValue for Result<YieldValues<I>, E>
     where
         I: Iterator<Item = T>,
-        T: IntoValue,
+        T: ArgList,
+        E: IntoError,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            self?.into_return_value()
+            let ruby = unsafe { Ruby::get_unchecked() };
+            self.map(|i| match i {
+                YieldValues::Iter(iter) => unsafe {
+                    do_yield_values_iter(iter);
+                    ruby.qnil().as_value()
+                },
+                YieldValues::Enumerator(e) => e.into_value_with(&ruby),
+            })
+            .map_err(|err| err.into_error(&ruby))
         }
     }
 
@@ -124,23 +149,25 @@ mod private {
         T: ArgList,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            match self {
-                YieldValues::Iter(iter) => unsafe {
-                    do_yield_values_iter(iter);
-                    Ok(Ruby::get_unchecked().qnil().as_value())
-                },
-                YieldValues::Enumerator(e) => Ok(unsafe { e.into_value_unchecked() }),
-            }
+            Ok::<Self, Error>(self).into_return_value()
         }
     }
 
-    impl<I, T> ReturnValue for Result<YieldValues<I>, Error>
+    impl<I, E> ReturnValue for Result<YieldSplat<I>, E>
     where
-        I: Iterator<Item = T>,
-        T: ArgList,
+        I: Iterator<Item = RArray>,
+        E: IntoError,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            self?.into_return_value()
+            let ruby = unsafe { Ruby::get_unchecked() };
+            self.map(|i| match i {
+                YieldSplat::Iter(iter) => unsafe {
+                    do_yield_splat_iter(iter);
+                    ruby.qnil().as_value()
+                },
+                YieldSplat::Enumerator(e) => e.into_value_with(&ruby),
+            })
+            .map_err(|err| err.into_error(&ruby))
         }
     }
 
@@ -149,22 +176,7 @@ mod private {
         I: Iterator<Item = RArray>,
     {
         fn into_return_value(self) -> Result<Value, Error> {
-            match self {
-                YieldSplat::Iter(iter) => unsafe {
-                    do_yield_splat_iter(iter);
-                    Ok(Ruby::get_unchecked().qnil().as_value())
-                },
-                YieldSplat::Enumerator(e) => Ok(unsafe { e.into_value_unchecked() }),
-            }
-        }
-    }
-
-    impl<I> ReturnValue for Result<YieldSplat<I>, Error>
-    where
-        I: Iterator<Item = RArray>,
-    {
-        fn into_return_value(self) -> Result<Value, Error> {
-            self?.into_return_value()
+            Ok::<Self, Error>(self).into_return_value()
         }
     }
 
@@ -178,9 +190,12 @@ mod private {
         }
     }
 
-    impl InitReturn for Result<(), Error> {
+    impl<E> InitReturn for Result<(), E>
+    where
+        E: IntoError,
+    {
         fn into_init_return(self) -> Result<(), Error> {
-            self
+            self.map_err(|err| err.into_error(&unsafe { Ruby::get_unchecked() }))
         }
     }
 
@@ -207,7 +222,7 @@ mod private {
     }
 }
 
-/// Trait implemented for function pointers that can be registed as Ruby
+/// Trait implemented for function pointers that can be registered as Ruby
 /// methods.
 ///
 /// While it is possible to directly write functions that will automatically
@@ -227,9 +242,9 @@ mod private {
 /// |     1 | `unsafe extern "C" fn(Value, Value) -> Value;`               |
 /// |     2 | `unsafe extern "C" fn(Value, Value, Value) -> Value;`        |
 /// |   ... | ...                                                          |
-/// |    16 | ...                                                          |
+/// |    15 | ...                                                          |
 ///
-/// note: for arity 0..=16 the number of arguments is 1 greater than the arity,
+/// note: for arity 0..=15 the number of arguments is 1 greater than the arity,
 /// due to the initial `self` argument.
 pub trait Method: private::Method {}
 
@@ -250,7 +265,7 @@ impl<T> Method for T where T: private::Method {}
 ///
 /// where `I` implements `Iterator<Item = T>` and `T` implements [`IntoValue`].
 ///
-/// When is `Err(magnus::Error)` returned to Ruby it will be conveted to and
+/// When is `Err(magnus::Error)` returned to Ruby it will be converted to and
 /// raised as a Ruby exception.
 ///
 /// [`Yield`], [`YieldValues`], and [`YieldSplat`] allow returning a Rust
@@ -271,7 +286,7 @@ impl<T> ReturnValue for T where T: private::ReturnValue {}
 /// * `()`
 /// * `Result<(), magnus::Error>`
 ///
-/// When is `Err(magnus::Error)` returned to Ruby it will be conveted to and
+/// When is `Err(magnus::Error)` returned to Ruby it will be converted to and
 /// raised as a Ruby exception.
 ///
 /// Note: functions without a specified return value will return `()`. `()`
@@ -289,7 +304,7 @@ impl<T> InitReturn for T where T: private::InitReturn {}
 ///
 /// where `T` implements [`IntoValue`].
 ///
-/// When is `Err(magnus::Error)` returned to Ruby it will be conveted to and
+/// When is `Err(magnus::Error)` returned to Ruby it will be converted to and
 /// raised as a Ruby exception.
 ///
 /// Note: functions without a specified return value will return `()`. `()`
@@ -367,18 +382,19 @@ where
 #[doc(hidden)]
 pub trait Block<Res>
 where
-    Self: Sized + FnMut(&[Value], Option<Proc>) -> Res,
+    Self: Sized + FnOnce(&Ruby, &[Value], Option<Proc>) -> Res,
     Res: BlockReturn,
 {
     #[inline]
     unsafe fn call_convert_value(
-        mut self,
+        self,
         argc: c_int,
         argv: *const Value,
         blockarg: Value,
     ) -> Result<Value, Error> {
+        let ruby = Ruby::get_unchecked();
         let args = slice::from_raw_parts(argv, argc as usize);
-        (self)(args, Proc::from_value(blockarg)).into_block_return()
+        (self)(&ruby, args, Proc::from_value(blockarg)).into_block_return()
     }
 
     #[inline]
@@ -398,7 +414,75 @@ where
 
 impl<Func, Res> Block<Res> for Func
 where
-    Func: FnMut(&[Value], Option<Proc>) -> Res,
+    Func: FnOnce(&Ruby, &[Value], Option<Proc>) -> Res,
+    Res: BlockReturn,
+{
+}
+
+/// Helper trait for wrapping a function with type conversions and error
+/// handling, when creating a thread.
+///
+/// See the [`Ruby::thread_create`] function.
+#[doc(hidden)]
+pub trait Thread<Res>
+where
+    Self: Sized + FnOnce(&Ruby) -> Res,
+    Res: BlockReturn,
+{
+    #[inline]
+    unsafe fn call_convert_value(self) -> Result<Value, Error> {
+        (self)(&Ruby::get_unchecked()).into_block_return()
+    }
+
+    #[inline]
+    unsafe fn call_handle_error(self) -> Value {
+        let res = match std::panic::catch_unwind(AssertUnwindSafe(|| self.call_convert_value())) {
+            Ok(v) => v,
+            Err(e) => Err(Error::from_panic(e)),
+        };
+        match res {
+            Ok(v) => v,
+            Err(e) => raise(e),
+        }
+    }
+}
+
+impl<Func, Res> Thread<Res> for Func
+where
+    Func: FnOnce(&Ruby) -> Res,
+    Res: BlockReturn,
+{
+}
+
+/// Helper trait for wrapping a function with type conversions and error
+/// handling, when calling [`Mutex::synchronize`](crate::Mutext::synchronize).
+#[doc(hidden)]
+pub trait Synchronize<Res>
+where
+    Self: Sized + FnOnce() -> Res,
+    Res: BlockReturn,
+{
+    #[inline]
+    unsafe fn call_convert_value(self) -> Result<Value, Error> {
+        (self)().into_block_return()
+    }
+
+    #[inline]
+    unsafe fn call_handle_error(self) -> Value {
+        let res = match std::panic::catch_unwind(AssertUnwindSafe(|| self.call_convert_value())) {
+            Ok(v) => v,
+            Err(e) => Err(Error::from_panic(e)),
+        };
+        match res {
+            Ok(v) => v,
+            Err(e) => raise(e),
+        }
+    }
+}
+
+impl<Func, Res> Synchronize<Res> for Func
+where
+    Func: FnOnce() -> Res,
     Res: BlockReturn,
 {
 }
@@ -687,7 +771,7 @@ macro_rules! method_n {
     }
 }
 
-seq!(N in 0..=16 {
+seq!(N in 0..=15 {
     method_n!(Method~N, RubyMethod~N, N);
 });
 
@@ -699,8 +783,8 @@ seq!(N in 0..=16 {
 /// [`define_method`](crate::module::Module::define_method).
 ///
 /// Ruby code implicitly always has a `self` parameter available. In the
-/// extention API this is passed explicitly. As a result there is always an
-/// extra `self` argument before the arguments explitly passed in Ruby, and the
+/// extension API this is passed explicitly. As a result there is always an
+/// extra `self` argument before the arguments explicitly passed in Ruby, and the
 /// number of Rust argument will be one more than the Ruby arity.
 ///
 /// The values `-2` and `-1` for `arity` have special meaning. Both indicate
@@ -732,20 +816,20 @@ seq!(N in 0..=16 {
 /// # Examples
 ///
 /// ```
-/// use magnus::{class, define_class, method, prelude::*, Error};
+/// use magnus::{method, prelude::*, Error, Ruby};
 ///
 /// fn rb_is_blank(rb_self: String) -> bool {
 ///     rb_self.contains(|c: char| !c.is_whitespace())
 /// }
 ///
 /// #[magnus::init]
-/// fn init() -> Result<(), Error> {
-///     let class = define_class("String", class::object())?;
+/// fn init(ruby: &Ruby) -> Result<(), Error> {
+///     let class = ruby.define_class("String", ruby.class_object())?;
 ///     class.define_method("blank?", method!(rb_is_blank, 0))?;
 ///     Ok(())
 /// }
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// # init().unwrap();
+/// # let cleanup = unsafe { magnus::embed::init() };
+/// # init(&cleanup).unwrap();
 /// ```
 #[macro_export]
 macro_rules! method {
@@ -1173,51 +1257,8 @@ macro_rules! method {
             $crate::Value,
         ) -> $crate::Value
     }};
-    ($name:expr, 16) => {{
-        unsafe extern "C" fn anon(
-            rb_self: $crate::Value,
-            a: $crate::Value,
-            b: $crate::Value,
-            c: $crate::Value,
-            d: $crate::Value,
-            e: $crate::Value,
-            f: $crate::Value,
-            g: $crate::Value,
-            h: $crate::Value,
-            i: $crate::Value,
-            j: $crate::Value,
-            k: $crate::Value,
-            l: $crate::Value,
-            m: $crate::Value,
-            n: $crate::Value,
-            o: $crate::Value,
-            p: $crate::Value,
-        ) -> $crate::Value {
-            use $crate::method::{Method16, RubyMethod16};
-            $name.call_handle_error(rb_self, a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p)
-        }
-        anon as unsafe extern "C" fn(
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-        ) -> $crate::Value
-    }};
     ($name:expr, $arity:expr) => {
-        compile_error!("arity must be an integer literal between -2..=16")
+        compile_error!("arity must be an integer literal between -2..=15")
     };
 }
 
@@ -1470,7 +1511,7 @@ macro_rules! function_n {
     }
 }
 
-seq!(N in 0..=16 {
+seq!(N in 0..=15 {
     function_n!(Function~N, RubyFunction~N, N);
 });
 
@@ -1483,7 +1524,7 @@ seq!(N in 0..=16 {
 /// [`define_method`](crate::module::Module::define_method).
 ///
 /// Ruby code implicitly always has a `self` parameter available. In the
-/// extention API this is passed explicitly. The wrapper this macro generates
+/// extension API this is passed explicitly. The wrapper this macro generates
 /// ignores that argument, and does not pass it to the wrapped function.
 ///
 /// The values `-2` and `-1` for `arity` have special meaning. Both indicate
@@ -1520,11 +1561,11 @@ seq!(N in 0..=16 {
 /// }
 ///
 /// #[magnus::init]
-/// fn init() {
-///     magnus::define_global_function("distance", magnus::function!(distance, 2));
+/// fn init(ruby: &magnus::Ruby) {
+///     ruby.define_global_function("distance", magnus::function!(distance, 2));
 /// }
-/// # let _cleanup = unsafe { magnus::embed::init() };
-/// # init();
+/// # let cleanup = unsafe { magnus::embed::init() };
+/// # init(&cleanup);
 /// ```
 #[macro_export]
 macro_rules! function {
@@ -1952,50 +1993,7 @@ macro_rules! function {
             $crate::Value,
         ) -> $crate::Value
     }};
-    ($name:expr, 16) => {{
-        unsafe extern "C" fn anon(
-            rb_self: $crate::Value,
-            a: $crate::Value,
-            b: $crate::Value,
-            c: $crate::Value,
-            d: $crate::Value,
-            e: $crate::Value,
-            f: $crate::Value,
-            g: $crate::Value,
-            h: $crate::Value,
-            i: $crate::Value,
-            j: $crate::Value,
-            k: $crate::Value,
-            l: $crate::Value,
-            m: $crate::Value,
-            n: $crate::Value,
-            o: $crate::Value,
-            p: $crate::Value,
-        ) -> $crate::Value {
-            use $crate::method::{Function16, RubyFunction16};
-            $name.call_handle_error(a, b, c, d, e, f, g, h, i, j, k, l, m, n, o, p)
-        }
-        anon as unsafe extern "C" fn(
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-            $crate::Value,
-        ) -> $crate::Value
-    }};
     ($name:expr, $arity:expr) => {
-        compile_error!("arity must be an integer literal between -2..=16")
+        compile_error!("arity must be an integer literal between -2..=15")
     };
 }
