@@ -1,21 +1,21 @@
 //! Types and functions for working with Ruby’s Hash class.
 
 use std::{
+    collections::BTreeMap,
     collections::HashMap,
+    convert::Infallible,
     fmt,
     hash::Hash,
     os::raw::{c_int, c_long},
     panic::AssertUnwindSafe,
 };
 
-#[cfg(ruby_gte_2_7)]
-use rb_sys::rb_hash_bulk_insert;
 #[cfg(ruby_gte_3_2)]
 use rb_sys::rb_hash_new_capa;
 use rb_sys::{
-    rb_check_hash_type, rb_hash_aref, rb_hash_aset, rb_hash_clear, rb_hash_delete, rb_hash_fetch,
-    rb_hash_foreach, rb_hash_lookup, rb_hash_lookup2, rb_hash_new, rb_hash_size, rb_hash_size_num,
-    rb_hash_update_by, ruby_value_type, VALUE,
+    rb_check_hash_type, rb_hash_aref, rb_hash_aset, rb_hash_bulk_insert, rb_hash_clear,
+    rb_hash_delete, rb_hash_fetch, rb_hash_foreach, rb_hash_lookup, rb_hash_lookup2, rb_hash_new,
+    rb_hash_size, rb_hash_size_num, rb_hash_update_by, ruby_value_type, VALUE,
 };
 
 use crate::{
@@ -125,6 +125,112 @@ impl Ruby {
     pub fn hash_new_capa(&self, n: usize) -> RHash {
         unsafe { RHash::from_rb_value_unchecked(rb_hash_new_capa(n as c_long)) }
     }
+
+    /// Create a new `RHash` from a Rust iterator.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use magnus::{rb_assert, Error, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_from_iter(["a", "b", "c"].into_iter().zip(1..4));
+    ///     rb_assert!(ruby, r#"hash == {"a" => 1, "b" => 2, "c" => 3}"#, hash);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    pub fn hash_from_iter<I, K, V>(&self, iter: I) -> RHash
+    where
+        I: IntoIterator<Item = (K, V)>,
+        K: IntoValue,
+        V: IntoValue,
+    {
+        self.hash_try_from_iter(iter.into_iter().map(Result::<_, Infallible>::Ok))
+            .unwrap()
+    }
+
+    /// Create a new `RHash` from a fallible Rust iterator.
+    ///
+    /// Returns `Ok(RHash)` on sucess or `Err(E)` with the first error
+    /// encountered.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use magnus::{rb_assert, Error, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_try_from_iter("a,1;b,2;c,3".split(';').map(|s| {
+    ///         s.split_once(',')
+    ///             .ok_or_else(|| Error::new(ruby.exception_runtime_error(), "bad format"))
+    ///     }))?;
+    ///     rb_assert!(
+    ///         ruby,
+    ///         r#"hash == {"a" => "1", "b" => "2", "c" => "3"}"#,
+    ///         hash
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    ///
+    /// ```
+    /// use magnus::{Error, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let err = ruby
+    ///         .hash_try_from_iter("a,1;b 2;c,3".split(';').map(|s| {
+    ///             s.split_once(',')
+    ///                 .ok_or_else(|| Error::new(ruby.exception_runtime_error(), "bad format"))
+    ///         }))
+    ///         .unwrap_err();
+    ///     assert_eq!(err.to_string(), "RuntimeError: bad format");
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    pub fn hash_try_from_iter<I, K, V, E>(&self, iter: I) -> Result<RHash, E>
+    where
+        I: IntoIterator<Item = Result<(K, V), E>>,
+        K: IntoValue,
+        V: IntoValue,
+    {
+        #[cfg(ruby_gte_3_2)]
+        pub fn hash_maybe_capa(n: usize) -> RHash {
+            unsafe { Ruby::get_unchecked() }.hash_new_capa(n)
+        }
+
+        #[cfg(ruby_lt_3_2)]
+        pub fn hash_maybe_capa(_: usize) -> RHash {
+            unsafe { Ruby::get_unchecked() }.hash_new()
+        }
+
+        let iter = iter.into_iter();
+        let (lower, _) = iter.size_hint();
+        let hash = if lower > 0 {
+            hash_maybe_capa(lower)
+        } else {
+            self.hash_new()
+        };
+        let mut buffer = [self.qnil().as_value(); 128];
+        let mut i = 0;
+        for r in iter {
+            let (k, v) = r?;
+            buffer[i] = self.into_value(k);
+            buffer[i + 1] = self.into_value(v);
+            i += 2;
+            if i >= buffer.len() {
+                i = 0;
+                hash.bulk_insert(&buffer).unwrap();
+            }
+        }
+        hash.bulk_insert(&buffer[..i]).unwrap();
+        Ok(hash)
+    }
 }
 
 /// A Value pointer to a RHash struct, Ruby's internal representation of Hash
@@ -173,6 +279,7 @@ impl RHash {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::RHash;
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -180,9 +287,10 @@ impl RHash {
     /// assert!(hash.is_empty());
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::hash_new` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[inline]
     pub fn new() -> RHash {
         get_ruby!().hash_new()
@@ -199,6 +307,7 @@ impl RHash {
     /// # Examples
     ///
     /// ```
+    /// # #![allow(deprecated)]
     /// use magnus::RHash;
     /// # let _cleanup = unsafe { magnus::embed::init() };
     ///
@@ -206,9 +315,10 @@ impl RHash {
     /// assert!(ary.is_empty());
     /// ```
     #[cfg_attr(
-        not(feature = "friendly-api"),
+        not(feature = "old-api"),
         deprecated(note = "please use `Ruby::hash_new_capa` instead")
     )]
+    #[cfg_attr(docsrs, doc(cfg(feature = "old-api")))]
     #[cfg(any(ruby_gte_3_2, docsrs))]
     #[cfg_attr(docsrs, doc(cfg(ruby_gte_3_2)))]
     #[inline]
@@ -223,12 +333,16 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{rb_assert, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{rb_assert, Error, Ruby};
     ///
-    /// let hash = RHash::new();
-    /// hash.aset("answer", 42).unwrap();
-    /// rb_assert!(r#"hash == {"answer" => 42}"#, hash);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_new();
+    ///     hash.aset("answer", 42)?;
+    ///     rb_assert!(ruby, r#"hash == {"answer" => 42}"#, hash);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn aset<K, V>(self, key: K, val: V) -> Result<(), Error>
     where
@@ -255,24 +369,27 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{prelude::*, rb_assert, RHash, RString, Symbol};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{prelude::*, rb_assert, Error, Ruby};
     ///
-    /// let hash = RHash::new();
-    /// hash.bulk_insert(&[
-    ///     Symbol::new("given_name").as_value(),
-    ///     RString::new("Arthur").as_value(),
-    ///     Symbol::new("family_name").as_value(),
-    ///     RString::new("Dent").as_value(),
-    /// ])
-    /// .unwrap();
-    /// rb_assert!(
-    ///     r#"hash == {given_name: "Arthur", family_name: "Dent"}"#,
-    ///     hash,
-    /// );
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_new();
+    ///     hash.bulk_insert(&[
+    ///         ruby.to_symbol("given_name").as_value(),
+    ///         ruby.str_new("Arthur").as_value(),
+    ///         ruby.to_symbol("family_name").as_value(),
+    ///         ruby.str_new("Dent").as_value(),
+    ///     ])
+    ///     .unwrap();
+    ///     rb_assert!(
+    ///         ruby,
+    ///         r#"hash == {given_name: "Arthur", family_name: "Dent"}"#,
+    ///         hash,
+    ///     );
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
-    #[cfg(any(ruby_gte_2_7, docsrs))]
-    #[cfg_attr(docsrs, doc(cfg(ruby_gte_2_7)))]
     pub fn bulk_insert<T>(self, slice: &[T]) -> Result<(), Error>
     where
         T: ReprValue,
@@ -290,18 +407,22 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, rb_assert, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{rb_assert, Error, RHash, Ruby};
     ///
-    /// let a: RHash = eval("{a: 1, b: 2}").unwrap();
-    /// let b: RHash = eval("{b: 3, c: 4}").unwrap();
-    /// a.update(b).unwrap();
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let a: RHash = ruby.eval("{a: 1, b: 2}")?;
+    ///     let b: RHash = ruby.eval("{b: 3, c: 4}")?;
+    ///     a.update(b)?;
     ///
-    /// // a is mutated, in case of conflicts b wins
-    /// rb_assert!("a == {a: 1, b: 3, c: 4}", a);
+    ///     // a is mutated, in case of conflicts b wins
+    ///     rb_assert!(ruby, "a == {a: 1, b: 3, c: 4}", a);
     ///
-    /// // b is unmodified
-    /// rb_assert!("b == {b: 3, c: 4}", b);
+    ///     // b is unmodified
+    ///     rb_assert!(ruby, "b == {b: 3, c: 4}", b);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     //
     // Implementation note: `rb_hash_update_by` takes a third optional argument,
@@ -319,38 +440,45 @@ impl RHash {
     /// Return the value for `key`, converting it to `U`.
     ///
     /// Returns hash's default if `key` is missing. See also
-    /// [`lookup`](RHash::lookup), [`get`](RHash::get), and
-    /// [`fetch`](RHash::fetch).
+    /// [`lookup`](RHash::lookup), [`lookup2`](RHash::lookup2),
+    /// [`get`](RHash::get), and [`fetch`](RHash::fetch).
     ///
     /// # Examples
     ///
     /// ```
-    /// use magnus::{value::Qnil, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{value::Qnil, Error, Ruby};
     ///
-    /// let hash = RHash::new();
-    /// hash.aset("answer", 42).unwrap();
-    /// assert_eq!(hash.aref::<_, i64>("answer").unwrap(), 42);
-    /// assert!(hash.aref::<_, Qnil>("missing").is_ok());
-    /// assert_eq!(hash.aref::<_, Option<i64>>("answer").unwrap(), Some(42));
-    /// assert_eq!(hash.aref::<_, Option<i64>>("missing").unwrap(), None);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_new();
+    ///     hash.aset("answer", 42)?;
+    ///     assert_eq!(hash.aref::<_, i64>("answer")?, 42);
+    ///     assert!(hash.aref::<_, Qnil>("missing").is_ok());
+    ///     assert_eq!(hash.aref::<_, Option<i64>>("answer")?, Some(42));
+    ///     assert_eq!(hash.aref::<_, Option<i64>>("missing")?, None);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(
-    ///     r#"
-    ///       hash = {"answer" => 42}
-    ///       hash.default = 0
-    ///       hash
-    ///     "#,
-    /// )
-    /// .unwrap();
-    /// assert_eq!(hash.aref::<_, i64>("answer").unwrap(), 42);
-    /// assert_eq!(hash.aref::<_, i64>("missing").unwrap(), 0);
-    /// assert_eq!(hash.aref::<_, i64>(()).unwrap(), 0);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(
+    ///         r#"
+    ///             hash = {"answer" => 42}
+    ///             hash.default = 0
+    ///             hash
+    ///         "#,
+    ///     )?;
+    ///     assert_eq!(hash.aref::<_, i64>("answer")?, 42);
+    ///     assert_eq!(hash.aref::<_, i64>("missing")?, 0);
+    ///     assert_eq!(hash.aref::<_, i64>(())?, 0);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn aref<T, U>(self, key: T) -> Result<U, Error>
     where
@@ -365,26 +493,30 @@ impl RHash {
     /// Return the value for `key`, converting it to `U`.
     ///
     /// Returns `nil` if `key` is missing. See also [`aref`](RHash::aref),
-    /// [`get`](RHash::get), and [`fetch`](RHash::fetch).
+    /// [`lookup2`](RHash::lookup2), [`get`](RHash::get), and
+    /// [`fetch`](RHash::fetch).
     ///
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, value::Qnil, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{value::Qnil, Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(
-    ///     r#"
-    ///       hash = {"answer" => 42}
-    ///       hash.default = 0
-    ///       hash
-    ///     "#,
-    /// )
-    /// .unwrap();
-    /// assert_eq!(hash.lookup::<_, i64>("answer").unwrap(), 42);
-    /// assert!(hash.lookup::<_, Qnil>("missing").is_ok());
-    /// assert_eq!(hash.lookup::<_, Option<i64>>("answer").unwrap(), Some(42));
-    /// assert_eq!(hash.lookup::<_, Option<i64>>("missing").unwrap(), None);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(
+    ///         r#"
+    ///             hash = {"answer" => 42}
+    ///             hash.default = 0
+    ///             hash
+    ///         "#,
+    ///     )?;
+    ///     assert_eq!(hash.lookup::<_, i64>("answer")?, 42);
+    ///     assert!(hash.lookup::<_, Qnil>("missing").is_ok());
+    ///     assert_eq!(hash.lookup::<_, Option<i64>>("answer")?, Some(42));
+    ///     assert_eq!(hash.lookup::<_, Option<i64>>("missing")?, None);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn lookup<T, U>(self, key: T) -> Result<U, Error>
     where
@@ -396,10 +528,58 @@ impl RHash {
             .and_then(TryConvert::try_convert)
     }
 
+    /// Return the value for `key` or the provided `default`, converting to `U`.
+    ///
+    /// Returns `default` if `key` is missing. See also [`aref`](RHash::aref),
+    /// [`lookup`](RHash::lookup), [`get`](RHash::get), and
+    /// [`fetch`](RHash::fetch).
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use magnus::{Error, RHash, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(
+    ///         r#"
+    ///             hash = {"foo" => 1, "bar" => nil}
+    ///             hash.default = 0
+    ///             hash
+    ///         "#,
+    ///     )?;
+    ///     assert_eq!(hash.lookup2::<_, _, i64>("foo", -1)?, 1);
+    ///     assert_eq!(hash.lookup2::<_, _, Option<i64>>("foo", -1)?, Some(1));
+    ///     assert_eq!(hash.lookup2::<_, _, Option<i64>>("bar", -1)?, None);
+    ///     assert_eq!(hash.lookup2::<_, _, Option<i64>>("baz", -1)?, Some(-1));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    pub fn lookup2<T, U, V>(self, key: T, default: U) -> Result<V, Error>
+    where
+        T: IntoValue,
+        U: IntoValue,
+        V: TryConvert,
+    {
+        let ruby = Ruby::get_with(self);
+        let key = ruby.into_value(key);
+        let default = ruby.into_value(default);
+        protect(|| unsafe {
+            Value::new(rb_hash_lookup2(
+                self.as_rb_value(),
+                key.as_rb_value(),
+                default.as_rb_value(),
+            ))
+        })
+        .and_then(TryConvert::try_convert)
+    }
+
     /// Return the value for `key` as a [`Value`].
     ///
     /// Returns `None` if `key` is missing. See also [`aref`](RHash::aref),
-    /// [`lookup`](RHash::lookup), and [`fetch`](RHash::fetch).
+    /// [`lookup`](RHash::lookup), [`lookup2`](RHash::lookup2), and
+    /// [`fetch`](RHash::fetch).
     ///
     /// Note: It is possible for very badly behaved key objects to raise an
     /// error during hash lookup. This is unlikely, and for the simplicity of
@@ -408,13 +588,17 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::RHash;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// let hash = RHash::new();
-    /// hash.aset("answer", 42).unwrap();
-    /// assert!(hash.get("answer").is_some());
-    /// assert!(hash.get("missing").is_none());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_new();
+    ///     hash.aset("answer", 42)?;
+    ///     assert!(hash.get("answer").is_some());
+    ///     assert!(hash.get("missing").is_none());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn get<T>(self, key: T) -> Option<Value>
     where
@@ -429,32 +613,36 @@ impl RHash {
             ))
         })
         .ok()
-        .and_then(|v| (!v.is_undef()).then(|| v))
+        .and_then(|v| (!v.is_undef()).then_some(v))
     }
 
     /// Return the value for `key`, converting it to `U`.
     ///
     /// Returns `Err` if `key` is missing. See also [`aref`](RHash::aref),
-    /// [`lookup`](RHash::lookup), and [`get`](RHash::get).
+    /// [`lookup`](RHash::lookup), [`lookup2`](RHash::lookup2), and
+    /// [`get`](RHash::get).
     ///
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(
-    ///     r#"
-    ///       hash = {"answer" => 42}
-    ///       hash.default = 0
-    ///       hash
-    ///     "#,
-    /// )
-    /// .unwrap();
-    /// assert_eq!(hash.fetch::<_, i64>("answer").unwrap(), 42);
-    /// assert!(hash.fetch::<_, i64>("missing").is_err());
-    /// assert_eq!(hash.fetch::<_, Option<i64>>("answer").unwrap(), Some(42));
-    /// assert!(hash.fetch::<_, Option<i64>>("missing").is_err());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(
+    ///         r#"
+    ///             hash = {"answer" => 42}
+    ///             hash.default = 0
+    ///             hash
+    ///         "#,
+    ///     )?;
+    ///     assert_eq!(hash.fetch::<_, i64>("answer")?, 42);
+    ///     assert!(hash.fetch::<_, i64>("missing").is_err());
+    ///     assert_eq!(hash.fetch::<_, Option<i64>>("answer")?, Some(42));
+    ///     assert!(hash.fetch::<_, Option<i64>>("missing").is_err());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn fetch<T, U>(self, key: T) -> Result<U, Error>
     where
@@ -474,12 +662,16 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, value::Qnil, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{value::Qnil, Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(r#"hash = {"answer" => 42}"#).unwrap();
-    /// assert_eq!(hash.delete::<_, i64>("answer").unwrap(), 42);
-    /// assert!(hash.delete::<_, Qnil>("answer").is_ok());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(r#"hash = {"answer" => 42}"#)?;
+    ///     assert_eq!(hash.delete::<_, i64>("answer")?, 42);
+    ///     assert!(hash.delete::<_, Qnil>("answer").is_ok());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn delete<T, U>(self, key: T) -> Result<U, Error>
     where
@@ -498,13 +690,17 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(r#"{"answer" => 42}"#).unwrap();
-    /// assert!(!hash.is_empty());
-    /// hash.clear().unwrap();
-    /// assert!(hash.is_empty());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(r#"{"answer" => 42}"#)?;
+    ///     assert!(!hash.is_empty());
+    ///     hash.clear()?;
+    ///     assert!(hash.is_empty());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn clear(self) -> Result<(), Error> {
         protect(|| unsafe { Value::new(rb_hash_clear(self.as_rb_value())) })?;
@@ -523,21 +719,24 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, r_hash::ForEach, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{r_hash::ForEach, Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4, "qux" => 8}"#).unwrap();
-    /// let mut found = None;
-    /// hash.foreach(|key: String, value: i64| {
-    ///     if value > 3 {
-    ///         found = Some(key);
-    ///         Ok(ForEach::Stop)
-    ///     } else {
-    ///         Ok(ForEach::Continue)
-    ///     }
-    /// })
-    /// .unwrap();
-    /// assert_eq!(found, Some(String::from("baz")));
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4, "qux" => 8}"#)?;
+    ///     let mut found = None;
+    ///     hash.foreach(|key: String, value: i64| {
+    ///         if value > 3 {
+    ///             found = Some(key);
+    ///             Ok(ForEach::Stop)
+    ///         } else {
+    ///             Ok(ForEach::Continue)
+    ///         }
+    ///     })?;
+    ///     assert_eq!(found, Some(String::from("baz")));
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn foreach<F, K, V>(self, mut func: F) -> Result<(), Error>
     where
@@ -559,8 +758,6 @@ impl RHash {
             let arg = &mut func as *mut F as VALUE;
             protect(|| {
                 let fptr = iter::<F, K, V> as unsafe extern "C" fn(VALUE, VALUE, VALUE) -> c_int;
-                #[cfg(ruby_lt_2_7)]
-                let fptr: unsafe extern "C" fn() -> c_int = std::mem::transmute(fptr);
                 rb_hash_foreach(self.as_rb_value(), Some(fptr), arg);
                 Ruby::get_with(self).qnil()
             })?;
@@ -583,13 +780,17 @@ impl RHash {
     /// ```
     /// use std::collections::HashMap;
     ///
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let r_hash: RHash = eval(r#"{"answer" => 42}"#).unwrap();
-    /// let mut hash_map = HashMap::new();
-    /// hash_map.insert(String::from("answer"), 42);
-    /// assert_eq!(r_hash.to_hash_map().unwrap(), hash_map);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let r_hash: RHash = ruby.eval(r#"{"answer" => 42}"#)?;
+    ///     let mut hash_map = HashMap::new();
+    ///     hash_map.insert(String::from("answer"), 42);
+    ///     assert_eq!(r_hash.to_hash_map()?, hash_map);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn to_hash_map<K, V>(self) -> Result<HashMap<K, V>, Error>
     where
@@ -604,6 +805,46 @@ impl RHash {
         Ok(map)
     }
 
+    /// Return `self` converted to a Rust [`BTreeMap`].
+    ///
+    /// This will only convert to a map of 'owned' Rust native types. The types
+    /// representing Ruby objects can not be stored in a heap-allocated
+    /// datastructure like a [`BTreeMap`] as they are hidden from the mark phase
+    /// of Ruby's garbage collector, and thus may be prematurely garbage
+    /// collected in the following sweep phase.
+    ///
+    /// Errors if the conversion of any key or value fails.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use std::collections::BTreeMap;
+    ///
+    /// use magnus::{Error, RHash, Ruby};
+    ///
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let r_hash: RHash = ruby.eval(r#"{"answer" => 42}"#)?;
+    ///     let mut hash_map = BTreeMap::new();
+    ///     hash_map.insert(String::from("answer"), 42);
+    ///     assert_eq!(r_hash.to_btree_map()?, hash_map);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
+    /// ```
+    pub fn to_btree_map<K, V>(self) -> Result<BTreeMap<K, V>, Error>
+    where
+        K: TryConvertOwned + Eq + Hash + Ord,
+        V: TryConvertOwned,
+    {
+        let mut map = BTreeMap::new();
+        self.foreach(|key, value| {
+            map.insert(key, value);
+            Ok(ForEach::Continue)
+        })?;
+        Ok(map)
+    }
+
     /// Convert `self` to a Rust vector of key/value pairs.
     ///
     /// This will only convert to a map of 'owned' Rust native types. The types
@@ -617,11 +858,15 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let r_hash: RHash = eval(r#"{"answer" => 42}"#).unwrap();
-    /// assert_eq!(r_hash.to_vec().unwrap(), vec![(String::from("answer"), 42)]);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let r_hash: RHash = ruby.eval(r#"{"answer" => 42}"#)?;
+    ///     assert_eq!(r_hash.to_vec()?, vec![(String::from("answer"), 42)]);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn to_vec<K, V>(self) -> Result<Vec<(K, V)>, Error>
     where
@@ -641,11 +886,15 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4}"#).unwrap();
-    /// assert_eq!(hash.size().to_i64(), 3);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4}"#)?;
+    ///     assert_eq!(hash.size().to_i64(), 3);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn size(self) -> Fixnum {
         unsafe { Fixnum::from_rb_value_unchecked(rb_hash_size(self.as_rb_value())) }
@@ -656,11 +905,15 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::{eval, RHash};
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, RHash, Ruby};
     ///
-    /// let hash: RHash = eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4}"#).unwrap();
-    /// assert_eq!(hash.len(), 3);
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash: RHash = ruby.eval(r#"{"foo" => 1, "bar" => 2, "baz" => 4}"#)?;
+    ///     assert_eq!(hash.len(), 3);
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn len(self) -> usize {
         unsafe { rb_hash_size_num(self.as_rb_value()) as usize }
@@ -671,13 +924,17 @@ impl RHash {
     /// # Examples
     ///
     /// ```
-    /// use magnus::RHash;
-    /// # let _cleanup = unsafe { magnus::embed::init() };
+    /// use magnus::{Error, Ruby};
     ///
-    /// let hash = RHash::new();
-    /// assert!(hash.is_empty());
-    /// hash.aset("answer", 42).unwrap();
-    /// assert!(!hash.is_empty());
+    /// fn example(ruby: &Ruby) -> Result<(), Error> {
+    ///     let hash = ruby.hash_new();
+    ///     assert!(hash.is_empty());
+    ///     hash.aset("answer", 42)?;
+    ///     assert!(!hash.is_empty());
+    ///
+    ///     Ok(())
+    /// }
+    /// # Ruby::init(example).unwrap()
     /// ```
     pub fn is_empty(self) -> bool {
         self.len() == 0
@@ -724,7 +981,28 @@ where
 {
 }
 
-#[cfg(feature = "friendly-api")]
+impl<K, V> IntoValue for BTreeMap<K, V>
+where
+    K: IntoValueFromNative,
+    V: IntoValueFromNative,
+{
+    fn into_value_with(self, handle: &Ruby) -> Value {
+        let hash = handle.hash_new();
+        for (k, v) in self {
+            let _ = hash.aset(k, v);
+        }
+        hash.into_value_with(handle)
+    }
+}
+
+unsafe impl<K, V> IntoValueFromNative for BTreeMap<K, V>
+where
+    K: IntoValueFromNative,
+    V: IntoValueFromNative,
+{
+}
+
+#[cfg(feature = "old-api")]
 impl<K, V> FromIterator<(K, V)> for RHash
 where
     K: IntoValue,
@@ -734,11 +1012,7 @@ where
     where
         I: IntoIterator<Item = (K, V)>,
     {
-        let hash = RHash::new();
-        for (k, v) in iter {
-            let _ = hash.aset(k, v);
-        }
-        hash
+        get_ruby!().hash_from_iter(iter)
     }
 }
 
