@@ -33,14 +33,16 @@ fn it_includes_backtrace_in_debug() {
         _ => panic!(),
     };
 
-    assert_eq!(
-        r#"RuntimeError: bang
-eval:3:in `foo'
-eval:7:in `bar'
-eval:11:in `baz'
-eval:15:in `qux'
-eval:18:in `<main>'
-"#,
-        format!("{:#?}", ex)
-    );
+    let message = format!("{:#?}", ex);
+    assert!(message.contains("RuntimeError: bang"));
+    assert!(message.contains("eval:3:in"));
+    assert!(message.contains("foo"));
+    assert!(message.contains("eval:7:in"));
+    assert!(message.contains("bar"));
+    assert!(message.contains("eval:11:in"));
+    assert!(message.contains("baz"));
+    assert!(message.contains("eval:15:in"));
+    assert!(message.contains("qux"));
+    assert!(message.contains("eval:18:in"));
+    assert!(message.contains("<main>"));
 }
