@@ -20,17 +20,7 @@ pub fn version() -> Version {
         major: MAJOR.parse().unwrap(),
         minor: MINOR.parse().unwrap(),
         patch: PATCH.parse().unwrap(),
-        pre: vec![],
-        build: vec![],
+        pre: Default::default(),
+        build: Default::default(),
     }
 }
-
-// We captured the build profile from build.rs and saved it in the cfg variable `neon_profile`.
-
-/// The current build profile (either `"release"` or `"debug"`).
-#[cfg(neon_profile = "release")]
-pub const BUILD_PROFILE: &str = "release";
-
-/// The current build profile (either `"release"` or `"debug"`).
-#[cfg(not(neon_profile = "release"))]
-pub const BUILD_PROFILE: &str = "debug";
